/**
 * @file   Game.cpp
 * @author yannick pencole
 * @date 14 Feb 2022
 * 
 **/
#include <unistd.h>
#include <iostream>
#include "Game.h"

namespace Oop
{
    void Game::onRightPressed()
    {
        // ici on met a jour le type de mouvement du heros 
        // sachant que l'utilisateur vient de presser la fleche droite
        std::cout << "fleche droite\n";
        _hero->setMovement(10,0);
    }
    
    void Game::onLeftPressed()
    {
        // ici on met a jour le type de mouvement du heros 
        // sachant que l'utilisateur vient de presser la fleche gauche
        std::cout << "fleche gauche\n";
        _hero->setMovement(-10,0);
    }    
    
    void Game::onUpPressed()
    {
        // ici on met a jour le type de mouvement du heros
        // sachant que l'utilisateur vient de presser la fleche haut
        std::cout << "fleche haut\n";
        _hero->setMovement(0,-10);
    }
    
    void Game::onDownPressed()
    {
        // ici on met a jour le type de mouvement du heros 
        // sachant que l'utilisateur vient de presser la fleche bas
        std::cout << "fleche bas\n"; 
        _hero->setMovement(0,10);
    }
    
    void Game::onSPressed()
    {
         // ici on met a jour le type de mouvement du heros 
        // sachant que l'utilisateur vient de presser s pour stopper le heros
        std::cout << "touche s\n"; 
        _hero->dontMove();
    }
    
    void Game::gameOver()
    {
        // Create a graphical text to display  
        sf::Font font;
        font.loadFromFile("DejaVuSans.ttf");
        sf::Text text("Game Over", font, 50); 
        text.setFillColor(sf::Color(255,0,0));
        text.setOutlineColor(sf::Color(255,240,240));
        text.setPosition(width()/2 - 100,height()/2 -30);
        sf::RectangleShape rectangle;
        rectangle.setPosition(text.getGlobalBounds().left,
                              text.getGlobalBounds().top);    
        rectangle.setSize(sf::Vector2f(text.getGlobalBounds().width,text.getGlobalBounds().height));
        rectangle.setFillColor(sf::Color(255,240,240));
        _window->draw(rectangle);
        _window->draw(text);
        _window->display();
        usleep(1000000);
        std::cout << "Fin du jeu, merci d'avoir participé\n";
        _window->close();
    }
     
    void Game::updateScene()
    {
       // ici nous devons mettre a jour toutes les positions des elements
       // qui doivent se deplacer sur la scene en vue de la redessiner
       _hero->updatePosition(); 
        for(size_t i = 0; i < _monsters.size(); ++i)
        {
            _monsters.at(i)->updatePosition();
            if(_hero->boundingBox().intersects(_monsters.at(i)->boundingBox()))
            {
               std::cout <<  _hero->decreasePoint() << " life points left.\n"; 
            }
        } 
       for(size_t i = 0; i < _settings.size(); ++i)
        {
            if(_hero->boundingBox().intersects(_settings.at(i)->boundingBox()))
            {
                _hero->decreasePoint();
                std::cout <<  _hero->decreasePoint() << " life points left.\n";
            }
        }
    }
    
    void Game::drawScene()
    {
        // ici nous devons redessiner tous les elements de la scene. (pour le moment 
        // cette scene  est vide car elle a ete nettoye dans la boucle de 'run')
        for(size_t i = 0; i < _settings.size(); ++i)
        {
//            sf::RectangleShape shape;
//            shape.setPosition(_settings.at(i)->boundingBox().left,_settings.at(i)->boundingBox().top);
//            shape.setSize(sf::Vector2f(_settings.at(i)->boundingBox().width,_settings.at(i)->boundingBox().height));
//            _window->draw(shape);
            _window->draw(*_settings.at(i));
        }  
        for(size_t i = 0; i < _monsters.size(); ++i)
        { 
            //sf::RectangleShape shape(_monsters.at(i)->boundingBox());
            //_window->draw(shape);
            _window->draw(*_monsters.at(i));
        }
        if(_hero!=nullptr)
        {
            if(!_hero->dead())
            {
//                sf::RectangleShape shape;
//                shape.setPosition(_hero->boundingBox().left,_hero->boundingBox().top);
//                shape.setSize(sf::Vector2f(_hero->boundingBox().width,_hero->boundingBox().height));
//                _window->draw(shape);
                _window->draw(*_hero);
                std::string lifePointMsg = "Life points: " + std::to_string(_hero->lifePoints());
                sf::Text text(lifePointMsg, _font, 12); 
                text.setFillColor(sf::Color(0,255,0));
                _window->draw(text);
            }
            else
            {
                gameOver();
            }
        }
    }
    
    
    Game::Game():_window(new sf::RenderWindow(sf::VideoMode(800, 300), "Jeu (par defaut)")),
            _settings(),
            _hero(nullptr),
            _monsters(),
            _width(800),
            _height(300),
            _font()
    { 
        _font.loadFromFile("DejaVuSans.ttf");
    }
    
    Game::Game(int width,int height, const std::string & title):
    _window(new sf::RenderWindow(sf::VideoMode(width, height), title)),
            _settings(),
            _hero(nullptr),
            _monsters(),
            _width(width),
            _height(height),
            _font()
    {
         _font.loadFromFile("DejaVuSans.ttf");
    }
  
  
  
    Game::~Game() 
    {
        if(_window != nullptr)
        {
            delete _window; _window = nullptr;
        }
        for(size_t i = 0; i < _settings.size(); ++i)
        {
            if(_settings.at(i)!=nullptr)
            {
                delete _settings.at(i);
                _settings.at(i) = nullptr;
            }
        }
        for(size_t i = 0; i < _monsters.size(); ++i)
        {
            if(_monsters.at(i)!=nullptr)
            {
                delete _monsters.at(i);
                _monsters.at(i) = nullptr;
            }
        }
    }
    
   
    
    void Game::run() 
    {
        std::cout << "Lancement du jeu..\n";
        _window->setKeyRepeatEnabled(false);
        // run the program as long as the window is open
        while (_window->isOpen()) {
        // check all the window's events that were triggered since the last iteration of the loop
            sf::Event event;
            while (_window->pollEvent(event)) {
                switch (event.type) {
                        // window closed
                    case sf::Event::Closed:
                    {
                        gameOver();
                        
                    }
                    break;
                        // key pressed
                    case sf::Event::KeyPressed:     
                        if (sf::Keyboard::isKeyPressed(sf::Keyboard::Left))
                        {
                            onLeftPressed();
                        }
                        else if (sf::Keyboard::isKeyPressed(sf::Keyboard::Right))
                        {
                            onRightPressed();
                        }
                        else if (sf::Keyboard::isKeyPressed(sf::Keyboard::Up))
                        {
                            onUpPressed();
                        }
                        else if (sf::Keyboard::isKeyPressed(sf::Keyboard::Down))
                        {
                            onDownPressed();
                        }
                        else if (sf::Keyboard::isKeyPressed(sf::Keyboard::S))
                        {
                            onSPressed();
                        }
                        else if (sf::Keyboard::isKeyPressed(sf::Keyboard::Escape))
                        {       
                             gameOver();
                           
                        }
                        break;
                    case sf::Event::KeyReleased:
                        break;
                    default:
                        break;
                }
            }
  
            // clear the window with black color
            _window->clear(sf::Color::Black);

            // frame rate
            usleep(100000);
            
            // update scene
            updateScene();
            
            // draw scene
            drawScene();

        
            // end the current frame
            _window->display();
        }
    }
    Hero * Game::newHero(sf::Texture * texture, 
                       sf::IntRect * rectSprite, 
                       float x, 
                       float y)
    {
        if(_hero!= nullptr)
        {
            delete _hero;
        }
        _hero = new Hero(texture,rectSprite,x,y,this);
        return _hero;
    }
    
    Setting * Game::newSetting(SettingType type, sf::Texture * texture, 
                             sf::IntRect * rectSprite, sf::IntRect * collider,
                             float x, 
                             float y)
    {
        _settings.push_back(new Setting(type,texture,rectSprite,collider,x,y,this));
        return _settings.back();
    }
    
    Monster * Game::newMonster(sf::Texture * texture, 
                       sf::IntRect * rectSprite, 
                       float x, 
                       float y)
    {
        _monsters.push_back(new Monster(texture,rectSprite,x,y,this));
        return _monsters.back();
    }
    
}


