/** 
 * @file:   Game.h
 * @author: Yannick Pencole
 * @date 14 Feb 2022
 **/

#ifndef GAME_H
#define GAME_H

#include <string>
#include <SFML/Graphics.hpp>
#include "Setting.h"
#include "Hero.h"
#include "Monster.h"

namespace Oop
{
   
    class Game {
    
    public:
        Game();
        Game(int width,int height, const std::string & title);
        
        float height() { return _height; }
        float width() { return _width; }

        virtual ~Game();
        void run();
        Setting * newSetting(SettingType type, sf::Texture * texture, 
                             sf::IntRect * rectSprite,
                             sf::IntRect * collider, 
                             float x, 
                             float y);
        Hero * newHero(sf::Texture * texture, 
                       sf::IntRect * rectSprite, 
                       float x, 
                       float y);
        
        Monster * newMonster(sf::Texture * texture, 
                       sf::IntRect * rectSprite, 
                       float x, 
                       float y);
        
    private:
        sf::RenderWindow * _window;
        std::vector<Setting *> _settings;
        Hero * _hero;
         std::vector<Monster *> _monsters;
         float _width;
         float _height;
         sf::Font _font;
       
    private:
        void gameOver();
        void onRightPressed();
        void onLeftPressed();
        void onUpPressed();
        void onDownPressed();
        void onCPressed();
        void onSPressed();
        void drawScene();
        void updateScene();
    };

}

#endif /* GAME_H */

